#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

from .. import parts

try: from selenium.webdriver.common.keys import Keys
except ImportError: Keys = None

class RipeRetailPart(parts.Part):

    def login(self, username, password):
        self.driver.get(self.login_url)

        form = self.driver.find_element_by_css_selector(".form")
        username_input = form.find_element_by_name("username")
        username_input.send_keys(username)
        password_input = form.find_element_by_name("password")
        password_input.send_keys(password)
        password_input.send_keys(Keys.ENTER)

    def login_and_redirect(self, username, password):
        self.login(username, password)

        self.waits.redirected_to(self.base_url)

    def select_size(self, size, gender = None, scale = None, open = True):
        """
        Opens the size selection window, selects the proper scale and size and
        applies that configuration by clicking 'Apply' and closing the window.

        Notice that if the "open" flag is unset the window is not opened.

        :type size: String
        :param size: The size to be picked.
        :type gender: String
        :param gender: The gender that is going to be picked.
        :type scale: String
        :param scale: The scale that is going to be picked.
        :type already_open: Boolean
        :param already_open: Whether the size modal is already open.
        """

        if open: self.interactions.click_when_possible(".size:not(.disabled) .button-size")

        if gender:
            self.interactions.click_when_possible(
                ".size .button-gender",
                condition = lambda element: element.text == gender
            )

        if scale:
            self.interactions.click_when_possible(
                ".size .button-scale",
                condition = lambda element: element.text == str(scale)
            )

        self.interactions.click_when_possible(
            ".size .button-size",
            condition = lambda element: element.text == str(size)
        )

        self.interactions.click_when_possible(".size .button.button-primary.button-apply")
        self.waits.is_not_visible(".size .modal")

    def set_part(
        self,
        brand,
        model,
        part,
        material,
        color,
        part_text = None,
        material_text = None,
        color_text = None
    ):
        """
        Makes a change to the customization of a part and checks that the pages
        mutates correctly, picking the right active parts, materials and colors,
        as well as properly switching the swatches.

        If the text parameters are passed an extra set of assertions are going
        to be performed to validate expected behaviour.

        :type brand: String
        :param brand: The brand of the model.
        :type model: String
        :param model: The model being customized.
        :type part: String
        :param part: The technical name of the part being changed.
        :type material: String
        :param material: The technical name of the material to use for the part.
        :type color: String
        :param color: The technical name of the color to use for the part.
        :type part_text: String
        :param part_text: The expected label for the part after clicking.
        :type material_text: String
        :param material_text: The expected label for the material after clicking.
        :type color_text: String
        :param color_text: The expected label for the color after clicking.
        """

        self.interactions.click_when_possible(
            ".pickers .button-part",
            condition = lambda e: e.text == part.upper()
        )
        if part_text: self.waits.text(".button-part.active", part_text)

        self.interactions.click_when_possible(".pickers .button-color[data-color='%s']" % color)

        if color_text: self.waits.text(".button-color.active", color_text)
        if material_text: self.waits.text(".button-material.active", material_text)

        self.waits.until(
            lambda d: self.assert_swatch(
                ".pickers .button-part.active .swatch > img",
                brand, model, material, color
            ),
            "Part swatch didn't have the expected image."
        )
        self.waits.until(
            lambda d: self.assert_swatch(
                ".pickers .button-color.active .swatch > img",
                brand, model, material, color
            ),
            "Color swatch didn't have the expected image."
        )

    def assert_swatch(self, selector, brand, model, material, color):
        """
        Checks that the img element identified by the selector points to the
        correct swatch. The correctness verification is performed by checking
        the "src" attribute of the element.

        This kind of assertion is critical to ensure proper responsiveness of
        the UI in accordance with part selection.

        :type selector: String
        :param selector: The selector for the img.
        :type brand: String
        :param brand: The brand of the swatch.
        :type model: String
        :param model: The model of the swatch.
        :type material: String
        :param material: The material the swatch should represent.
        :type color: String
        :param color: The color being shown in the shown.
        """

        element = self.waits.element(selector)
        src = element.get_attribute("src")
        expected_params = [
            "brand=%s" % brand,
            "model=%s" % model,
            "material=%s" % material,
            "color=%s" % color
        ]

        is_correct = all(expected_param in src for expected_param in expected_params)

        if not is_correct:
            raise AssertionError(
                "Expected '%s' (src of '%s') to contain '%s'." %\
                (src, selector, expected_params)
            )

    @property
    def retail_url(self):
        base_url = appier.conf("BASE_URL", "https://ripe-retail-test.platforme.com")
        return appier.conf("RETAIL_URL", base_url)

    @property
    def login_url(self):
        return "%s/login" % self.retail_url

    @property
    def logout_url(self):
        return "%s/logout" % self.retail_url



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

from . import drivers

from .. import test_cases

class InteractiveTestCase(test_cases.TestCase):

    def __init__(self, *args, **kwargs):
        test_cases.TestCase.__init__(self, *args, **kwargs)
        self.options = kwargs
        self.driver = None
        self.timeout = appier.conf("TIMEOUT", 15, cast = int)
        self.timeout = self.options.get("timeout", self.timeout)

    def before(self):
        test_cases.TestCase.before(self)
        self.driver = self.load_driver()

    def after(self):
        if self.driver:
            self.driver.stop()
            self.driver = None
        test_cases.TestCase.after(self)

    def load_driver(self, start = True):
        driver_s = appier.conf("DRIVER", "selenium")
        driver_s = self.options.get("driver", driver_s)
        driver = getattr(drivers, driver_s.capitalize() + "Driver")(self)
        if start: driver.start()
        return driver



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

from .. import parts

try: from selenium.webdriver.common.keys import Keys
except ImportError: Keys = None

class RipeIdPart(parts.Part):

    def login(self, username = None, password = None):
        username = appier.conf("RIPE_ID_USERNAME", username)
        password = appier.conf("RIPE_ID_PASSWORD", password)
        if not username or not password:
            self.owner.skip(message = "No RIPE ID credentials available")

        self.driver.get(self.base_url)

        self.interactions.click_when_possible(".button-platforme")
        self.waits.redirected_to(self.login_url)

        form = self.driver.find_element_by_css_selector(".form")
        username_input = form.find_element_by_name("username")
        username_input.send_keys(username)
        password_input = form.find_element_by_name("password")
        password_input.send_keys(password)
        password_input.send_keys(Keys.ENTER)

        self.interactions.click_when_possible(".form .button-blue")

    def login_and_redirect(self, username = None, password = None):
        self.login(username = username, password = password)
        self.waits.redirected_to(self.home_url)

    @property
    def id_url(self):
        return appier.conf("ID_URL", "https://id.platforme.com")

    @property
    def login_url(self):
        return "%s/admin/signin" % self.id_url



#!/usr/bin/python
# -*- coding: utf-8 -*-

def test_fullname(test):
    instance = test.__self__
    module = instance.__class__.__module__
    if module == None or module == str.__class__.__module__:
        return instance.__class__.__name__ + "." + test.__name__
    return module + "." + instance.__class__.__name__ + "." + test.__name__



#!/usr/bin/python
# -*- coding: utf-8 -*-

from .. import parts

try: from selenium.common.exceptions import ElementClickInterceptedException
except ImportError: ElementClickInterceptedException = None

try: from selenium.common.exceptions import ElementNotVisibleException
except ImportError: ElementNotVisibleException = None

try: from selenium.common.exceptions import WebDriverException
except ImportError: WebDriverException = None

class InteractionsPart(parts.Part):

    def try_click(self, element, focus = True):
        try:
            self.driver.click(element, focus = focus)
            return element
        except (
            ElementClickInterceptedException,
            ElementNotVisibleException,
            WebDriverException
        ) as exception:
            self.logger.debug("Element is not \"clickable\" because: %s" % exception)
            return None

    def click_when_possible(self, selector, condition = None):
        """
        Clicks an element when possible, which happens when that element is both
        visible and clickable.

        :type selector: String
        :param selector: The selector for the element to click.
        :type condition: Function
        :param condition: The filter the selected element mustpass to be clickable.
        :rtype Element
        :return The clicked element if there's any othewise an invalid value.
        """

        element = self.waits.is_visible(selector, condition = condition)
        return self.waits.until(
            lambda d: self.try_click(element),
            "Element '%s' found but never became clickable" % selector
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

from .. import parts

try: from selenium.webdriver.support.ui import WebDriverWait
except ImportError: WebDriverWait = None

try: from selenium.common.exceptions import StaleElementReferenceException
except ImportError: StaleElementReferenceException = None

class WaitsPart(parts.Part):

    def __init__(self, owner):
        parts.Part.__init__(self, owner)
        self.wait = WebDriverWait(self.driver, self.timeout)

    def wrap_method(self, method, *args):
        """
        Wraps the method being waited on to tolerate some exceptions since
        in most cases these are transitory conditions that shouldn't break
        the test.

        :type method: Function
        :param method: The method being ran.
        :type args: List
        :param args: The arguments provided to the method.
        :rtype Function
        :return: The method wrapped on a try-catch for StaleElementReferenceException.
        """

        try:
            return method(*args)
        except (StaleElementReferenceException, AssertionError) as e:
            self.logger.debug("Got exception while waiting: %s" % e)
            return None

    def until(self, method, message = None):
        return self.wait.until(
            lambda *args: self.wrap_method(method, *args),
            message = message
        )

    def redirected_to(self, url):
        return self.until(
           lambda d: self.assertions.at_url(url),
            message = "Expecting the page to be '%s' but is '%s'" % (
                url,
                self.driver.current_url
            )
        )

    def element(self, selector, condition = None):
        return self.until(
            lambda d: self.assertions.exists(selector, condition = condition),
            "Element '%s' not found" % selector
        )

    def elements(self, selector, condition = None):
        return self.until(
            lambda d: self.assertions.exists_multiple(selector, condition = condition),
            "Elements '%s' not found" % selector
        )

    def text(self, selector, text):
        return self.until(
            lambda d: self.assertions.has_text(selector, text),
            "Element '%s' with text '%s' not found" % (selector, text)
        )

    def is_visible(self, selector, condition = None):
        return self.until(
            lambda d: self.assertions.is_visible(selector, condition = condition),
            "Element '%s' is not visible" % selector
        )

    def is_not_visible(self, selector, condition = None):
        return self.until(
            lambda d: self.assertions.is_not_visible(selector, condition = condition),
            "Element '%s' is visible" % selector
        )
